@@ -28,11 +28,11 @@ from .errors import (
 )
 from .response import SMTPResponse
 from .smtp import SMTP
-from .status import SMTPStatus
+from .typing import SMTPStatus
 
 
 __title__ = "aiosmtplib"
-__version__ = "1.1.7"
+__version__ = "2.0.0"
 __author__ = "Cole Maclean"
 __license__ = "MIT"
 __copyright__ = "Copyright 2022 Cole Maclean"
