@@ -7,10 +7,7 @@ from typing import NamedTuple
 __all__ = ("SMTPResponse",)
 
 
-BaseResponse = NamedTuple("BaseResponse", [("code", int), ("message", str)])
-
-
-class SMTPResponse(BaseResponse):
+class SMTPResponse(NamedTuple):
     """
     NamedTuple of server response code and server response message.
 
@@ -26,10 +23,11 @@ class SMTPResponse(BaseResponse):
 
     """
 
-    __slots__ = ()
+    code: int
+    message: str
 
     def __repr__(self) -> str:
-        return "({self.code}, {self.message})".format(self=self)
+        return f"({self.code}, {self.message})"
 
     def __str__(self) -> str:
-        return "{self.code} {self.message}".format(self=self)
+        return f"{self.code} {self.message}"
