@@ -3,6 +3,7 @@ Tests covering SMTP configuration options.
 """
 import asyncio
 import socket
+import ssl
 
 import pytest
 
@@ -12,74 +13,90 @@ from aiosmtplib import SMTP
 pytestmark = pytest.mark.asyncio()
 
 
-async def test_tls_context_and_cert_raises():
+async def test_tls_context_and_cert_raises(
+    client_tls_context: ssl.SSLContext,
+) -> None:
     with pytest.raises(ValueError):
-        SMTP(use_tls=True, client_cert="foo.crt", tls_context=True)
+        SMTP(use_tls=True, client_cert="foo.crt", tls_context=client_tls_context)
 
 
-async def test_tls_context_and_cert_to_connect_raises():
-    client = SMTP(use_tls=True, tls_context=True)
+async def test_tls_context_and_cert_to_connect_raises(
+    client_tls_context: ssl.SSLContext,
+) -> None:
+    client = SMTP(use_tls=True, tls_context=client_tls_context)
 
     with pytest.raises(ValueError):
         await client.connect(client_cert="foo.crt")
 
 
-async def test_tls_context_and_cert_to_starttls_raises(smtp_client, smtpd_server):
+async def test_tls_context_and_cert_to_starttls_raises(
+    smtp_client: SMTP,
+    smtpd_server: asyncio.AbstractServer,
+    client_tls_context: ssl.SSLContext,
+) -> None:
     async with smtp_client:
         with pytest.raises(ValueError):
-            await smtp_client.starttls(client_cert="test.cert", tls_context=True)
+            await smtp_client.starttls(
+                client_cert="test.cert", tls_context=client_tls_context
+            )
 
 
-async def test_use_tls_and_start_tls_raises():
+async def test_use_tls_and_start_tls_raises() -> None:
     with pytest.raises(ValueError):
         SMTP(use_tls=True, start_tls=True)
 
 
-async def test_use_tls_and_start_tls_to_connect_raises():
+async def test_use_tls_and_start_tls_to_connect_raises() -> None:
     client = SMTP(use_tls=True)
 
     with pytest.raises(ValueError):
         await client.connect(start_tls=True)
 
 
-async def test_socket_and_hostname_raises():
+async def test_socket_and_hostname_raises() -> None:
     with pytest.raises(ValueError):
         SMTP(hostname="example.com", sock=socket.socket(socket.AF_INET))
 
 
-async def test_socket_and_port_raises():
+async def test_socket_and_port_raises() -> None:
     with pytest.raises(ValueError):
         SMTP(port=1, sock=socket.socket(socket.AF_INET))
 
 
-async def test_socket_and_socket_path_raises():
+async def test_socket_and_socket_path_raises() -> None:
     with pytest.raises(ValueError):
         SMTP(socket_path="/tmp/test", sock=socket.socket(socket.AF_INET))  # nosec
 
 
-async def test_hostname_and_socket_path_raises():
+async def test_hostname_and_socket_path_raises() -> None:
     with pytest.raises(ValueError):
         SMTP(hostname="example.com", socket_path="/tmp/test")  # nosec
 
 
-async def test_port_and_socket_path_raises():
+async def test_port_and_socket_path_raises() -> None:
     with pytest.raises(ValueError):
         SMTP(port=1, socket_path="/tmp/test")  # nosec
 
 
-async def test_config_via_connect_kwargs(hostname, smtpd_server_port):
+async def test_config_via_connect_kwargs(
+    bind_address: str, unused_tcp_port: int, hostname: str, smtpd_server_port: int
+) -> None:
     client = SMTP(
         hostname="",
         use_tls=True,
+        start_tls=None,
         port=smtpd_server_port + 1,
-        source_address="example.com",
+        local_hostname="example.com",
     )
 
-    source_address = socket.getfqdn()
+    local_hostname = "smtp.example.com"
+    source_address = (bind_address, unused_tcp_port)
     await client.connect(
         hostname=hostname,
         port=smtpd_server_port,
         use_tls=False,
+        start_tls=False,
+        local_hostname=local_hostname,
         source_address=source_address,
     )
     assert client.is_connected
@@ -87,7 +104,9 @@ async def test_config_via_connect_kwargs(hostname, smtpd_server_port):
     assert client.hostname == hostname
     assert client.port == smtpd_server_port
     assert client.use_tls is False
+    assert client.local_hostname == local_hostname
     assert client.source_address == source_address
+    assert client._start_tls_on_connect is False
 
     await client.quit()
 
@@ -98,8 +117,11 @@ async def test_config_via_connect_kwargs(hostname, smtpd_server_port):
     ids=["plaintext", "tls", "starttls"],
 )
 async def test_default_port_on_connect(
-    event_loop, bind_address, use_tls, start_tls, expected_port
-):
+    bind_address: str,
+    use_tls: bool,
+    start_tls: bool,
+    expected_port: int,
+) -> None:
     client = SMTP()
 
     try:
@@ -115,9 +137,10 @@ async def test_default_port_on_connect(
 
 
 async def test_connect_hostname_takes_precedence(
-    event_loop, hostname, smtpd_server_port
-):
-    client = SMTP(hostname="example.com", port=smtpd_server_port)
+    hostname: str,
+    smtpd_server_port: int,
+) -> None:
+    client = SMTP(hostname="example.com", port=smtpd_server_port, start_tls=False)
     await client.connect(hostname=hostname)
 
     assert client.hostname == hostname
@@ -125,8 +148,11 @@ async def test_connect_hostname_takes_precedence(
     await client.quit()
 
 
-async def test_connect_port_takes_precedence(event_loop, hostname, smtpd_server_port):
-    client = SMTP(hostname=hostname, port=17)
+async def test_connect_port_takes_precedence(
+    hostname: str,
+    smtpd_server_port: int,
+) -> None:
+    client = SMTP(hostname=hostname, port=17, start_tls=False)
     await client.connect(port=smtpd_server_port)
 
     assert client.port == smtpd_server_port
@@ -134,8 +160,13 @@ async def test_connect_port_takes_precedence(event_loop, hostname, smtpd_server_
     await client.quit()
 
 
-async def test_connect_timeout_takes_precedence(hostname, smtpd_server_port):
-    client = SMTP(hostname=hostname, port=smtpd_server_port, timeout=0.66)
+async def test_connect_timeout_takes_precedence(
+    hostname: str,
+    smtpd_server_port: int,
+) -> None:
+    client = SMTP(
+        hostname=hostname, port=smtpd_server_port, timeout=0.66, start_tls=False
+    )
     await client.connect(timeout=0.99)
 
     assert client.timeout == 0.99
@@ -143,35 +174,70 @@ async def test_connect_timeout_takes_precedence(hostname, smtpd_server_port):
     await client.quit()
 
 
-async def test_connect_source_address_takes_precedence(hostname, smtpd_server_port):
+async def test_connect_source_address_takes_precedence(
+    bind_address: str,
+    unused_tcp_port: int,
+    hostname: str,
+    smtpd_server_port: int,
+) -> None:
     client = SMTP(
-        hostname=hostname, port=smtpd_server_port, source_address="example.com"
+        hostname=hostname,
+        port=smtpd_server_port,
+        start_tls=False,
+        source_address=("example.com", 444),
     )
-    await client.connect(source_address=socket.getfqdn())
+    await client.connect(source_address=(bind_address, unused_tcp_port))
 
-    assert client.source_address != "example.com"
+    assert client.source_address == (bind_address, unused_tcp_port)
 
     await client.quit()
 
 
-async def test_connect_event_loop_takes_precedence(
-    event_loop, event_loop_policy, hostname, smtpd_server_port
-):
-    init_loop = event_loop_policy.new_event_loop()
-    with pytest.warns(DeprecationWarning):
-        client = SMTP(hostname=hostname, port=smtpd_server_port, loop=init_loop)
+async def test_connect_local_hostname_takes_precedence(
+    hostname: str,
+    smtpd_server_port: int,
+) -> None:
+    client = SMTP(
+        hostname=hostname,
+        port=smtpd_server_port,
+        start_tls=False,
+        local_hostname="foo.com",
+    )
+    await client.connect(local_hostname="example.com")
 
-    with pytest.warns(DeprecationWarning):
-        await client.connect(loop=event_loop)
-
-    assert init_loop is not event_loop
-    assert client.loop is event_loop
+    assert client.local_hostname == "example.com"
 
     await client.quit()
 
 
-async def test_connect_use_tls_takes_precedence(hostname, smtpd_server_port):
-    client = SMTP(hostname=hostname, port=smtpd_server_port, use_tls=True)
+async def test_connect_deprecated_source_address(
+    hostname: str,
+    smtpd_server_port: int,
+) -> None:
+    client = SMTP(
+        hostname=hostname,
+        port=smtpd_server_port,
+        start_tls=False,
+    )
+    with pytest.warns(DeprecationWarning):
+        await client.connect(source_address="example.com")  # type: ignore
+
+    assert client.local_hostname == "example.com"
+
+    await client.quit()
+
+
+async def test_connect_use_tls_takes_precedence(
+    hostname: str,
+    smtpd_server_port: int,
+    client_tls_context: ssl.SSLContext,
+) -> None:
+    client = SMTP(
+        hostname=hostname,
+        port=smtpd_server_port,
+        use_tls=True,
+        tls_context=client_tls_context,
+    )
 
     await client.connect(use_tls=False)
 
@@ -180,8 +246,17 @@ async def test_connect_use_tls_takes_precedence(hostname, smtpd_server_port):
     await client.quit()
 
 
-async def test_connect_validate_certs_takes_precedence(hostname, smtpd_server_port):
-    client = SMTP(hostname=hostname, port=smtpd_server_port, validate_certs=False)
+async def test_connect_validate_certs_takes_precedence(
+    hostname: str,
+    smtpd_server_port: int,
+    client_tls_context: ssl.SSLContext,
+) -> None:
+    client = SMTP(
+        hostname=hostname,
+        port=smtpd_server_port,
+        validate_certs=False,
+        tls_context=client_tls_context,
+    )
 
     await client.connect(validate_certs=True)
 
@@ -190,13 +265,18 @@ async def test_connect_validate_certs_takes_precedence(hostname, smtpd_server_po
     await client.quit()
 
 
-async def test_connect_certificate_options_take_precedence(hostname, smtpd_server_port):
+async def test_connect_certificate_options_take_precedence(
+    hostname: str,
+    smtpd_server_port: int,
+    client_tls_context: ssl.SSLContext,
+) -> None:
     client = SMTP(
         hostname=hostname,
         port=smtpd_server_port,
         client_cert="test",
         client_key="test",
         cert_bundle="test",
+        start_tls=False,
     )
 
     await client.connect(client_cert=None, client_key=None, cert_bundle=None)
@@ -209,8 +289,11 @@ async def test_connect_certificate_options_take_precedence(hostname, smtpd_serve
 
 
 async def test_connect_tls_context_option_takes_precedence(
-    hostname, smtpd_server_port, client_tls_context, server_tls_context
-):
+    hostname: str,
+    smtpd_server_port: int,
+    client_tls_context: ssl.SSLContext,
+    server_tls_context: ssl.SSLContext,
+) -> None:
     client = SMTP(
         hostname=hostname, port=smtpd_server_port, tls_context=server_tls_context
     )
@@ -223,11 +306,16 @@ async def test_connect_tls_context_option_takes_precedence(
 
 
 async def test_starttls_certificate_options_take_precedence(
-    hostname, smtpd_server_port, valid_cert_path, valid_key_path
-):
+    hostname: str,
+    smtpd_server_port: int,
+    ca_cert_path: str,
+    valid_cert_path: str,
+    valid_key_path: str,
+) -> None:
     client = SMTP(
         hostname=hostname,
         port=smtpd_server_port,
+        start_tls=False,
         validate_certs=False,
         client_cert="test1",
         client_key="test1",
@@ -244,44 +332,33 @@ async def test_starttls_certificate_options_take_precedence(
     await client.starttls(
         client_cert=valid_cert_path,
         client_key=valid_key_path,
-        cert_bundle=valid_cert_path,
+        cert_bundle=ca_cert_path,
         validate_certs=True,
     )
 
     assert client.client_cert == valid_cert_path
     assert client.client_key == valid_key_path
-    assert client.cert_bundle == valid_cert_path
+    assert client.cert_bundle == ca_cert_path
     assert client.validate_certs is True
 
     await client.quit()
 
 
-async def test_loop_kwarg_deprecation_warning_init(event_loop):
+async def test_source_address_deprecation_warning_init() -> None:
     with pytest.warns(DeprecationWarning):
-        client = SMTP(loop=event_loop)
+        client = SMTP(source_address="example.com")  # type: ignore
 
-    assert client.loop == event_loop
+    assert client.local_hostname == "example.com"
 
 
-async def test_loop_kwarg_deprecation_warning_connect(
-    event_loop, hostname, smtpd_server_port, smtpd_server
-):
-    client = SMTP(hostname=hostname, port=smtpd_server_port)
-
-    with pytest.warns(DeprecationWarning):
-        await client.connect(loop=event_loop)
-
-    assert client.loop == event_loop
-
-
-async def test_hostname_newline_raises_error():
+async def test_hostname_newline_raises_error() -> None:
     with pytest.raises(ValueError):
         SMTP(hostname="localhost\r\n")
 
 
-async def test_source_address_newline_raises_error():
+async def test_local_hostname_newline_raises_error() -> None:
     with pytest.raises(ValueError):
         SMTP(
             hostname="localhost",
-            source_address="localhost\r\nRCPT TO: <hacker@hackers.org>",
+            local_hostname="localhost\r\nRCPT TO: <hacker@hackers.org>",
         )
