@@ -26,11 +26,9 @@ pytestmark = [
 ]
 
 
-async def test_starttls_gmail():
+async def test_auto_starttls_gmail() -> None:
     client = SMTP(hostname="smtp.gmail.com", port=587, use_tls=False)
     await client.connect(timeout=1.0)
-    await client.ehlo()
-    await client.starttls(validate_certs=False)
     response = await client.ehlo()
 
     assert response.code == SMTPStatus.completed
@@ -41,9 +39,9 @@ async def test_starttls_gmail():
         await client.login("test", "test")
 
 
-async def test_qq_login():
+async def test_qq_login() -> None:
     client = SMTP(hostname="smtp.qq.com", port=587, use_tls=False)
-    await client.connect(timeout=2.0)
+    await client.connect(timeout=2.0, start_tls=False)
     await client.ehlo()
     await client.starttls(validate_certs=False)
 
@@ -51,7 +49,7 @@ async def test_qq_login():
         await client.login("test", "test")
 
 
-async def test_office365_auth_send():
+async def test_office365_auth_send() -> None:
     message = EmailMessage()
     message["From"] = "user@mydomain.com"
     message["To"] = "somebody@example.com"
@@ -69,14 +67,14 @@ async def test_office365_auth_send():
         )
 
 
-async def test_office365_skip_login():
+async def test_office365_skip_login() -> None:
     message = EmailMessage()
     message["From"] = "user@mydomain.com"
     message["To"] = "somebody@example.com"
     message["Subject"] = "Hello World!"
     message.set_content("Sent via aiosmtplib")
 
-    smtp_client = SMTP("smtp.office365.com", 587)
+    smtp_client = SMTP("smtp.office365.com", 587, start_tls=False)
     await smtp_client.connect()
     await smtp_client.starttls()
     # skip login, which is required
